"""
Viscoelastic indentation.
This code computes pressure distribution for indentation of a viscoelastic material by a rigid sphere.
The indentation is prescribed by a given velocity of the sphere.

V.A. Yastrebov, MINES Paris - PSL, CNRS, Evry/Paris, France, 2024.

License: BSD 3 clause
"""

import numpy as np
import matplotlib.pyplot as plt
import scipy.integrate as integrate

# Indentation evolution in time
def delta(t,v):
    return v*t
# Indentation rate
def Ddelta_Dt(t,v):
    dt = 1e-10
    return (delta(t+dt,v)-delta(t-dt,v))/(2*dt)

N       = 100
rmax    = .03
# For visualization needs only (maximal pressure)
pmax    = 0.05
# Radius of the indenter
R = 1
# Shear modulus for infinite time
mu0 = 1
# Shear modulus for short time
mu1 = 1
# Poisson's ratio for incompressible material
nu = 0.5
# Relaxation time
tau = 1.
# Maximum indentation
delta_max = 0.0003
# Indentation velocity
v = 1e-4
# Compute the time of indentation
tmax = delta_max/v
# Number of time steps
Nt = 201
dt = tmax/Nt

time = 0
r = np.linspace(0,rmax,N)
start_time = np.zeros(N)
pressure = np.zeros(N)

# Animate the process
fig,ax = plt.subplots()
plt.ion()
plt.show()

start_time = np.zeros(N)
start_time = r**2/(v*R)

def elastic_pressure(r,R,mu,delta):
    pressure = np.zeros_like(r)
    for i,x in enumerate(r):
        if x > np.sqrt(delta*R):
            pressure[i] = 0
        else:
            pressure[i] = np.sqrt(delta*R-x**2)*mu/(1-nu)*2/np.pi
    return pressure

for it in range(Nt):
    ax.clear()
    ax.set_xlim(0,rmax)
    ax.grid()
    time += dt
    plt.xlabel("Radial coordinate, $r$")
    plt.ylabel("Pressure, $p$")
    plt.title("Time/$t_{\\max}$ = "+str(round(time/tmax*100,2))+" %")

    # Plot elastic results
    ax.plot(r,elastic_pressure(r,R,mu0,delta(time,v)), "--", color="k",label="Elastic, slow")
    ax.plot(r,elastic_pressure(r,R,mu1,delta(time,v)), "-", color="k",label="Elastic, fast")
    # Integrate the viscoelastic solution
    for i in range(N):
        if start_time[i] < time:
            pressure[i] = 2/(np.pi*(1-nu)) * integrate.quad(lambda tt: ( mu0 + (mu1-mu0)*np.exp(-(time-tt)/tau) ) * Ddelta_Dt(tt,v) / 2 / np.sqrt(delta(tt,v)*R-r[i]**2),a=start_time[i],b=time)[0]

    # Plot viscoelastic results
    ax.set_ylim(0,pmax)
    ax.plot(r,pressure,"-",color="r",lw=2,label="Viscoelastic solution")
    plt.legend()
    fig.canvas.draw()
    fig.canvas.flush_events()

plt.show()

fig.savefig("IndentationPressure_v_{:.2e}.png" .format(v))

